# -*- coding: utf-8 -*-
'''
Copyright (C) 2008 Thomas Holder, http://sf.net/users/speleo3/
Distributed under the terms of the GNU General Public License v2
'''

import math, sys, os

programfiles = os.getenv('PROGRAMFILES')
if programfiles:
	sys.path.append(programfiles + '\\Inkscape\\share\\extensions')
sys.path.append('/usr/share/inkscape/extensions')
sys.path.append('/Applications/Inkscape.app/Contents/Resources/extensions')

import inkex

class Scalebar:
	def __init__(self, scale, dpi = 90):
		mag = int(math.log10(scale))

		if mag < 2:
			units = 'cm'
			factor = 10 ** mag
		elif mag < 5:
			units = 'm'
			factor = 10 ** (mag - 2)
		else:
			units = 'km'
			factor = 10 ** (mag - 5)

		step = scale / (10 ** mag)

		if step < 2:
			step = 1
		elif step < 5:
			step = 2
		else:
			step = 5

		steplength = dpi / 2.54 / scale * step * 10 ** mag

		d = "m%f,0 h-%f v7 h%f z " % (2 * steplength, steplength, steplength) * 3
		d = d.replace(' ', ' m-%f,0 ' % steplength, 1)

		self.g = inkex.etree.Element('g')
		self.g.set('style', 'font-size:10px;text-anchor:middle;font-family:sans-serif')

		node = inkex.etree.Element('path')
		node.set('d', d)
		node.set('style', 'fill:black;stroke:none')
		self.g.append(node)

		node = inkex.etree.Element('path')
		node.set('d', 'M0,0 h%f M0,7 h%f' % (steplength * 5, steplength * 5))
		node.set('style', 'fill:none;stroke:black;stroke-width:0.5')
		self.g.append(node)

		for i in range(6):
			node = inkex.etree.Element('text')
			node.set('x', str(steplength * i))
			node.set('y', '18')
			node.text = str(step * i * factor)
			if i == 5:
				node.text += units
			self.g.append(node)

		node = inkex.etree.Element('text')
		node.set('y', '-5')
		node.set('style', 'text-anchor:start')
		node.text = u"Maßstab 1:" + str(scale)
		self.g.append(node)
	
	def get_tree(self):
		return self.g
	
	def get_xml(self):
		return inkex.etree.tostring(self.g)

class InsertScalebar(inkex.Effect):
	def __init__(self):
		inkex.Effect.__init__(self)
		self.OptionParser.add_option("--scale",
						action="store", type="int",
						dest="scale", default=100,
						help="Scale")
		self.OptionParser.add_option("--dpi",
						action="store", type="int",
						dest="dpi", default=90,
						help="DPI")
	
	def get_current_layer(self):
		layer = self.current_layer
		while True:
			groupmode = layer.get(inkex.addNS('groupmode', 'inkscape'))
			if groupmode == 'layer':
				break
			parent = layer.getparent()
			if parent == None:
				break
			layer = parent
		return layer
	
	def effect(self):
		g = Scalebar(self.options.scale, self.options.dpi).get_tree()

		g.set('transform', 'translate' + str(self.view_center))
		self.get_current_layer().append(g)

if __name__ == '__main__':
	e = InsertScalebar()
	e.affect()

